/****************************************************************************/
// Function: Automatic Relay-off of a 3D-Printer after cooling down hotend.
// Details: When pushing the button,3D-Printer ist supplied for Time_Wait seconds.
// If nothing happens, 3D-Printer will be Relayed off.
// If the current exceed Idle_Current, printer remains suppied, because printer was Relayed on.
// Alternatively you can push the button to extend the remaining on-time.
// Now printer will be supervised. If current exceed Work_Current in a window of Time_Retrigger, 
// Controller ist detecting a working print job. If time runs out without one short Work_Current in this period,
// Controller leads to a cool down phas of Time_Cooldown. In this period, you can leave this shutdown state by
// pressing the button. There you will put the controller into the idle-state, for example to start a new printing job.
// If no button is pressed, printer will be disconnected from main, after time runs out.
// This program based on: 
// Hardware: Grove - Electricity Sensor
// Date: Jan 19,2013
// by www.seeedstudio.com
//
// The program is Common Creative Licenced CC SA, so only my name has to remain. All other stuff you can do.
// V1.0, inital release by Jennifer Sirtl, 11.9.2021
//

#define ANZEIGE
#ifdef ANZEIGE
  #include <Arduino.h>
  #include <U8x8lib.h>
  #include <Wire.h>
  U8X8_SSD1306_128X64_NONAME_SW_I2C u8x8(/* clock=*/ SCL, /* data=*/ SDA, /* reset=*/ U8X8_PIN_NONE);   // OLEDs without Reset of the Display  
#endif

#define Current_Sensor A0 // Analog input pin that sensor is attached to
#define Relay 5 //Pin for relay stage
#define Button 2//Pin for wake Button
#define Rotary1 3
#define Rotary2 4

const float Idle_Current=100; //half of Current, Printer in idle consumes
const float Work_Current=2000; //half of Current Printer in work consumes (peak)
const unsigned long Time_Wait=300; //Time to wait for reaction (witch on printer)
const unsigned long Time_Retrigger=90; //Time to 3 Pulses should occur
const unsigned long Time_Cooldown=600; //Time to cooldown hotend

enum StateEnum{Idle,WaitState,PrinterOn, PrinterPrint,PrinterFinish};

unsigned long Timer,DTimer;
float CurrentPeak; //current
float effective_value; //effective current
StateEnum State;


#ifdef ANZEIGE
  //Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);
#endif

void setup() {
  Serial.begin(9600);
  
#ifdef ANZEIGE
  u8x8.begin();
  u8x8.setPowerSave(0);
  u8x8.setFont(u8x8_font_chroma48medium8_r);
  u8x8.refreshDisplay();    // only required for SSD1606/7  
  delay(2000);  
#endif
  
  pinMode(Current_Sensor, INPUT);
  pinMode(Relay, OUTPUT);
  pinMode(Button, INPUT_PULLUP);
  State=Idle;
  digitalWrite(Relay,LOW);
  Serial.println(F("3DPrintSafe, V211128"));
  Serial.println(F("Current-Sensor: A0, Relay:4, Pushbutton:7 (tears to GND)"));
  Serial.println(F("I:Idle"));

#ifdef ANZEIGE
  u8x8.setFont(u8x8_font_chroma48medium8_r);
  u8x8.setInverseFont(1);
  u8x8.drawString(0,0,"AutoOff2  211128");
  u8x8.setInverseFont(0);
  u8x8.setFont(u8x8_font_chroma48medium8_r);
  u8x8.drawString(0,1,"                ");
  u8x8.setFont(u8x8_font_chroma48medium8_r);
  u8x8.drawString(0,2,"                ");
  u8x8.setFont(u8x8_font_chroma48medium8_r);
  u8x8.drawString(0,3,"Idle            ");
  u8x8.setFont(u8x8_font_chroma48medium8_r);
  u8x8.drawString(0,4,"Number:         ");
  u8x8.setFont(u8x8_font_chroma48medium8_r);
  u8x8.drawString(0,5,"Curr:           ");
  u8x8.setFont(u8x8_font_chroma48medium8_r);
  u8x8.drawString(0,6,"Time:           ");
  u8x8.setFont(u8x8_font_chroma48medium8_r);
  u8x8.drawString(0,7,"                ");
  u8x8.refreshDisplay();    // only required for SSD1606/7  
#endif
  
}

void loop() {
  
  CurrentPeak = getCurrentPeak();
  Serial.print(F("Ipk:"));
  Serial.print(CurrentPeak);
  Serial.println(F("mA"));

  u8x8.setFont(u8x8_font_chroma48medium8_r);
  u8x8.drawString(0,5,"Curr:          ");
  u8x8.setFont(u8x8_font_chroma48medium8_r);
  u8x8.drawString(0,6,"Time:          ");
  u8x8.setCursor(5,5);
  u8x8.print(CurrentPeak,2);
  u8x8.setCursor(5,6);
  u8x8.print(DTimer);
  

  switch (State){
    //--------------
    case Idle:
      if (digitalRead(Button)==LOW){
        State=WaitState;
        Timer=millis();
        do{
          delay(50);
        }
        while (digitalRead(Button)==HIGH);
        digitalWrite(Relay,HIGH);
        Serial.println(F("T:Idle->Wait by Button"));
        u8x8.setFont(u8x8_font_chroma48medium8_r);
        u8x8.drawString(0,3,"Wait for Button");

      }
      break;
      
    //--------------
    case WaitState:
      DTimer=(Time_Wait*1000-(millis()-Timer))/1000;
      if (millis()-Timer >=Time_Wait*1000){
        State=Idle;
        digitalWrite(Relay,LOW);
        Serial.println(F("T:Wait->Idle by Timer_Wait"));
        u8x8.setFont(u8x8_font_chroma48medium8_r);
        u8x8.drawString(0,3,"Idle Timer Wait");        
      }
      else{
        if (CurrentPeak>=Idle_Current){
          State=PrinterOn;
          Timer=millis();
         Serial.println(F("T:Wait->PrinterOn by Idle_Current"));
         u8x8.setFont(u8x8_font_chroma48medium8_r);
         u8x8.drawString(0,3,"Printer Idle   ");          
        }
        if (digitalRead(Button)==LOW){
          Timer=millis();//Retrigger timer
         Serial.println(F("X:Wait Retrigger by Button"));
         u8x8.setFont(u8x8_font_chroma48medium8_r);
         u8x8.drawString(0,3,"Retrigger Butto");        

        }
      }
      break;
    //--------------
    case PrinterOn:
      {
        if (CurrentPeak>=Work_Current){
          State=PrinterPrint;
          Timer=millis(); 
          Serial.println(F("T:PrinterOn->PrinterPrint by Work_Current"));
          u8x8.setFont(u8x8_font_chroma48medium8_r);
          u8x8.drawString(0,3,"Printer working");        
        }
        else{
          if (CurrentPeak<Idle_Current){
            State=WaitState;
            Timer=millis();
            Serial.println(F("T:PriterOn->Wait by Idle_Current"));
            u8x8.setFont(u8x8_font_chroma48medium8_r);
            u8x8.drawString(0,3,"Wait Idle Curr.");        
          }
        }
      }
      break; 
    //--------------      
    case PrinterPrint:
      {
        if (CurrentPeak>=Work_Current){
          Timer=millis();//Retrigger Timer
          Serial.println(F("X:PrinterPrint Retrigger by Work_Current"));
          u8x8.setFont(u8x8_font_chroma48medium8_r);
          u8x8.drawString(0,3,"Retriggered Cur");        
          
        }
        DTimer=(Time_Retrigger*1000-(millis()-Timer))/1000;
        if (millis()-Timer>=Time_Retrigger*1000){
          State=PrinterFinish;
          Timer=millis();
          Serial.println(F("T:PrinterPrint->PrinterFinish by Time_Retrigger"));       
          u8x8.setFont(u8x8_font_chroma48medium8_r);
          u8x8.drawString(0,3,"Printer finish ");        
        }
      }
      break;
    //--------------      
    case PrinterFinish:
      {
        DTimer=(Time_Cooldown*1000-(millis()-Timer))/1000;
        if (millis()-Timer>=Time_Cooldown*1000){
          State=Idle;
          digitalWrite(Relay,LOW); 
          Serial.println(F("T:PrinterFinish->Idle by Time_Cooldown"));
        u8x8.setFont(u8x8_font_chroma48medium8_r);
        u8x8.drawString(0,3,"Cooldown       ");        
        }  
        if (CurrentPeak>=Work_Current){
          State=PrinterPrint;
          Timer=millis();  
          Serial.println(F("T:PrinterFinish->PrinterPrint by Work_Current"));       
          u8x8.setFont(u8x8_font_chroma48medium8_r);
          u8x8.drawString(0,3,"Retriggered Cur");        
        }
      }
      break; 
    //--------------
    default: break;
  }
}


float getCurrentPeak()
{
int sensorValue; //value read from the sensor
int sensorMax = 0;
uint32_t start_time = millis();
while((millis()-start_time) < 1000)//sample for 1000ms
{
sensorValue = analogRead(Current_Sensor);
if (sensorValue > sensorMax){
  sensorMax = sensorValue;
}
}
return (float)sensorMax/1024*5/800*2000000-12.21;
}
